/*
  RC4Test.cpp
*/
#define MAX_BLOB 50
BYTE bPlainText1[MAX_BLOB];
BYTE bPlainText2[MAX_BLOB];
BYTE bCipherText1[MAX_BLOB];
BYTE bCipherText2[MAX_BLOB];
BYTE bKeyStream[MAX_BLOB];
BYTE bKey[MAX_BLOB];

//////////////////////////////////////////////////////////////////
// Setup  nastaven dvou prostch text a ifrovacho kle.
void Setup() {
    ZeroMemory(bPlainText1, MAX_BLOB);
    ZeroMemory(bPlainText2, MAX_BLOB);
    ZeroMemory(bCipherText1, MAX_BLOB);
    ZeroMemory(bCipherText2, MAX_BLOB);
    ZeroMemory(bKeyStream, MAX_BLOB);
    ZeroMemory(bKey, MAX_BLOB);

    strncpy(reinterpret_cast<char*>(bPlainText1),
        "Nazdar Pepo, potkme se v est na e.", MAX_BLOB-1);

    strncpy(reinterpret_cast<char*>(bPlainText2),
        "Ped okny Akademika jsou mlkai.", MAX_BLOB-1);

    strncpy(reinterpret_cast<char*>(bKey),
        GetKeyFromUser(), MAX_BLOB-1);	//  Extern funkce 
}

//////////////////////////////////////////////////////////////////
// Encrypt  zaifruje blok dat algoritmem RC4.
void Encrypt(LPBYTE bKey, 
             LPBYTE bPlaintext,
             LPBYTE bCipherText, 
             DWORD dwHowMuch) {
                 HCRYPTPROV hProv;
                 HCRYPTKEY  hKey;
                 HCRYPTHASH hHash;

    /*
      Cel to funguje takhle:
      Zjistme popisova k poskytovateli ifrovn.
      Vytvome przdn objekt hae.
      Do objektu hae provedeme haovn kle.
      Pomoc hae z kroku 3 odvodme ifrovac kl. Ten se
      pak ukld tak v algoritmu, kter provd vlastn ifrovn.
      Pomoc ifrovacho kle z kroku 4 zaifrujeme prost text.
    */

    DWORD dwBuff = dwHowMuch;
    CopyMemory(bCipherText, bPlaintext, dwHowMuch);
    if (!CryptAcquireContext(&hProv, NULL, NULL, PROV_RSA_FULL, 
                             CRYPT_VERIFYCONTEXT))
        throw;
    if (!CryptCreateHash(hProv, CALG_MD5, 0, 0, &hHash))
        throw;
    if (!CryptHashData(hHash, bKey, MAX_BLOB, 0))
        throw;
    if (!CryptDeriveKey(hProv, CALG_RC4, hHash, 
                        CRYPT_EXPORTABLE, 
                        &hKey))
        throw;
    if (!CryptEncrypt(hKey, 0, TRUE, 0, 
                      bCipherText, 
                      &dwBuff, 
                      dwHowMuch))
        throw;

    if (hKey)  CryptDestroyKey(hKey);
    if (hHash) CryptDestroyHash(hHash);
    if (hProv) CryptReleaseContext(hProv, 0);
}

void main() {
    Setup();

    // Zaifrujeme oba prost texty pomoc kle bKey.
    try {
        Encrypt(bKey, bPlainText1, bCipherText1, MAX_BLOB);
        Encrypt(bKey, bPlainText2, bCipherText2, MAX_BLOB);
    } catch (...) {
        printf("Chyba - %d", GetLastError());
        return;
    }

    // Te nsleduje trocha "arovn".
    // Vezmeme kad bajt ze znmho ifrovanho nebo prostho textu.
    for (int i = 0; i < MAX_BLOB; i++) {
        BYTE c1 = bCipherText1[i];        //  Bajty prvnho ifrovanho 
        BYTE p1 = bPlainText1[i];         //  Bajty prvnho prostho
        BYTE k1 = c1 ^ p1;                //  Zde zjistme bajty kle
        BYTE p2 = k1 ^ bCipherText2[i];   //  Bajty druhho ifrovanho

        // Vypeme kad bajt druh zprvy.
        printf("%c", p2);
    }
}
