/*
    CleanCanon.cpp
*/
#include "stdafx.h"
#include "atlrx.h"
#include "strsafe.h"
#include <new>

enum errCanon {
    ERR_CANON_NO_ERROR = 0,
    ERR_CANON_INVALID_FILENAME,
    ERR_CANON_INVALID_PATH,
    ERR_CANON_NOT_A_FILE,
    ERR_CANON_NO_FILE,
    ERR_CANON_NO_PATH,
    ERR_CANON_TOO_BIG,
    ERR_CANON_NO_MEM};

errCanon GetCanonicalFileName(LPCTSTR szFilename, 
                              LPCTSTR szDir,
                              LPTSTR  *pszNewFilename) {

    // KROK 1
    // Mus zadat cestu a mus bt krat ne MAX_PATH

    if (szDir == NULL)
        return ERR_CANON_NO_PATH;

    size_t cchDirLen = 0;
    if (StringCchLength(szDir, MAX_PATH, &cchDirLen) != S_OK ||
            cchDirLen > MAX_PATH)
        return ERR_CANON_TOO_BIG;

    *pszNewFilename = NULL;
    LPTSTR szTempFullDir = NULL;
    HANDLE hFile = NULL;

    errCanon err = ERR_CANON_NO_ERROR;

    try {
        // KROK 2 
        // Zkontrolujeme, je-li nzev souboru platn (alfanum '.' 0-4 alfanum)
        // Velikost psmen nerozliujeme
        CAtlRegExp<> reFilename, reDirname;
        CAtlREMatchContext<> mc;
        reFilename.Parse(_T"^\\a+\\.\\a\\a?\\a?\\a?$", FALSE);
        if (!reFilename.Match(szFilename, &mc))
            throw ERR_CANON_INVALID_FILENAME;

        reDirname.Parse(_T"^\\c:\\\\[a-z0-9\\\\]+$", FALSE);
        if (!reDirname.Match(szDir, &mc))
            throw ERR_CANON_INVALID_FILENAME;

        size_t cFilename = lstrlen(szFilename);
        size_t cDir = lstrlen(szDir);

        // Velikost novho doasnho bufferu, navc msto pro '\'.
        size_t cNewFilename = cFilename + cDir + 1;

        // KROK 3
        // Kontrola, nen-li nzev souboru pli dlouh.
        if (cNewFilename > MAX_PATH)
            throw ERR_CANON_TOO_BIG;

        // Alokace pamti pro nov nzev souboru.
        // Potme i s koncovm '\0'.
        LPCTSTR szPrefix = _T("\\\\?\\");
        size_t cchPrefix = lstrlen(szPrefix);
        size_t cchTempFullDir = cNewFilename + 1 + cchPrefix;
        szTempFullDir = new TCHAR[cchTempFullDir];
        if (szTempFullDir == NULL)
            throw ERR_CANON_NO_MEM;
      
        // KROK 4 
        // Zetzen nzvu adrese a souboru. 
        // Na zatek doplnme \\?\, take OS zpracuje znaky doslovn
        // a neprovede dal interpretaci/kanonizaci.
        if (StringCchPrintf(szTempFullDir,
                            cchTempFullDir,
                            _T("%s%s\\%s"),
                            szPrefix,
                            szDir,
                            szFilename) != S_OK)
            throw ERR_CANON_INVALID_FILENAME;

        // KROK 5 
        // Nast plnou cestu; 
        // potme i s .. a koncovou '.' a mezerami
        TCHAR szFullPathName [MAX_PATH + 1];
        LPTSTR szFilenamePortion = NULL; 
        DWORD dwFullPathLen = 
            GetFullPathName(szTempFullDir,
                            MAX_PATH,
                            szFullPathName,
                            &szFilenamePortion);
        if (dwFullPathLen > MAX_PATH)
            throw ERR_CANON_NO_MEM;

        // KROK 6 
        // Zjistit dlouh nzev souboru.
        if (GetLongPathName(szFullPathName, 
                            szFullPathName, 
                            MAX_PATH) == 0) {
            errCanon errName = ERR_CANON_TOO_BIG;
            switch(GetLastError()) {
                case ERROR_FILE_NOT_FOUND :
                         errName = ERR_CANON_NO_FILE;
                         break;

                case ERROR_NOT_READY :
                case ERROR_PATH_NOT_FOUND :
                         errName = ERR_CANON_NO_PATH;
                         break;

                default : break;
            }

            throw errName;
        }

        // KROK 7
        // Je to soubor nebo zazen?
        hFile = CreateFile(szFullPathName,
                           0, 0, NULL,
                           OPEN_EXISTING,
                       SECURITY_SOOS_PRESENT | SECURITY_IDENTIFICATION
                           NULL);
        if (hFile == INVALID_HANDLE_VALUE)
            throw ERR_CANON_NO_SUCH_FILE;
           
        if (GetFileType(hFile) != FILE_TYPE_DISK)
            throw ERR_CANON_NOT_A_FILE;
      
        // Vechno vypad v podku!
        // Volajc mus provst delete [] pszNewFilename.
        const size_t cNewFilename = lstrlen(szFullPathName)+1;
        *pszNewFilename = new TCHAR[cNewFilename];
        if (*pszNewFilename != NULL)
            StringCchCopy(*pszNewFilename, cNewFilename, szFullPathName);
        else
            err = ERR_CANON_NO_MEM;

    } catch(errCanon e) {
        err = e;
    } catch(std::bad_alloc a) {
        err = ERR_CANON_NO_MEM;
    } 

    delete [] szTempFullDir;
    if (hFile) CloseHandle(hFile);

    return err;
}


int _tmain(int argc, _TCHAR* argv[]) {
    errCanon err = ERR_CANON_NO_ERROR;

    if (argc > 1) {
        _TCHAR *pNewFilename = NULL;
        err = GetCanonicalFileName(argv[1],"c:\\temp",&pNewFilename);
        
        if (err == ERR_CANON_NO_ERROR) {
            printf("%s je platn\n",pNewFilename);
            delete [] pNewFilename;
        } else {
            printf("%s je neplatn (%d)\n",argv[1],err);
        }
    }

    return err;
}

