/* 
    RoundTrip.cpp : Definuje vstupn bod konzolov aplikace.
*/

#include "stdafx.h"

/*
  CheckRoundTrip
  Vrac TRUE, pokud se dan etzec mezi Unicode a danou kdovou
  strnkou poda pevst zpt do pvodn podoby; jinak vrac FALSE.
*/

BOOL CheckRoundTrip(
                    DWORD uiCodePage,
                    LPWSTR wszString) 
{

    BOOL fStatus = TRUE;
    BYTE *pbTemp = NULL;
    WCHAR *pwcTemp = NULL;

    try {
        // Zjistme, je-li dlka etzce < MAX_STRING_LEN
        // Korektn obsloume i przdn etzce 
        const size_t MAX_STRING_LEN = 200;
        size_t cchCount = 0;
        if (!SUCCEEDED(StringCchLength(wszString, 
                       MAX_STRING_LEN, &cchCount))) 
            throw FALSE;

        pbTemp = new BYTE[MAX_STRING_LEN];
        pwcTemp = new WCHAR[MAX_STRING_LEN];
        if (!pbTemp || !pwcTemp) {
            printf("CHYBA: nen voln pam!\n");
            throw FALSE;
        }

        ZeroMemory(pbTemp,MAX_STRING_LEN * sizeof(BYTE));
        ZeroMemory(pwcTemp,MAX_STRING_LEN * sizeof(WCHAR));

        // Pevod z Unicode do poadovan kdov strnky.
        int rc =  WideCharToMultiByte( uiCodePage,
            0,
            wszString,
            -1,
            (LPSTR)pbTemp,
            MAX_STRING_LEN,
            NULL,
            NULL );
        if (!rc) {
            printf("CHYBA: WC2MB chyba = %d, CodePage = %d, 
                    String = %ws\n",
                GetLastError(), uiCodePage, wszString);
            throw FALSE;
        }

        // Pevod ze zadan kdov strnky zpt do Unicode.
        rc = MultiByteToWideChar( uiCodePage,
            0,
            (LPSTR)pbTemp,
            -1,
            pwcTemp,
            MAX_STRING_LEN / sizeof(WCHAR) );
        if (!rc) {
            printf("CHYBA: MB2WC chyba = %d, CodePage = %d, 
                    String = %ws\n",
                GetLastError(), uiCodePage, wszString);
            throw FALSE;
        }

        // Zjistme dlku pvodnho etzce Unicode
        // a zkontrolujeme, jestli je shodn s dlkou po pevodu.
        size_t Length = 0;
        StringCchLength(wszString, MAX_STRING_LEN,&Length);
        if (Length+1 != rc) {
            printf("Dlka %d != rc %d\n", Length, rc);
            throw FALSE;
        }

        // Nyn porovnme cel pvodn etzec Unicode s pevedenm
        // etzcem a ovme, jestli jsou shodn.
        for (size_t ctr = 0; ctr < Length; ctr++) {
            if (pwcTemp[ctr] != wszString[ctr])
                throw FALSE;
        }
    } catch (BOOL iErr) {
        fStatus = iErr;
    }

    if (pbTemp)  delete [] pbTemp;
    if (pwcTemp) delete [] pwcTemp;

    return (fStatus);
}

int _cdecl main(
                int argc,
                char* argv[])
{
    LPWSTR s1 = L"\x00a9MicrosoftCorp";          // Copyright
    LPWSTR s2 = L"To\x221e&Beyond";              // Nekoneno

    printf("1252 Copyright = %d\n", CheckRoundTrip(1252, s1));
    printf("437  Copyright = %d\n", CheckRoundTrip(437, s1));
    printf("1252 Infinity  = %d\n", CheckRoundTrip(1252, s2));
    printf("437  Infinity  = %d\n", CheckRoundTrip(437, s2));

    return (1);
}
