/* TestClient.cpp
  Tato aplikace ukazuje dsledky rznch bezpenostnch nastaven DCOM 
  Budeme pouvat star typy voln 
*/

#define _WIN32_WINNT 0x0500
#include "../DCOM_Security.h"
#include <objbase.h>
#include <stdio.h>

void DumpSecuritySettings(DWORD AuthNSvc, 
						  DWORD AuthZSvc, 
						  OLECHAR* PriName, 
						  DWORD AuthLevel, 
						  DWORD ImpLevel)
{
	// vpis autentizan sluby
	switch(AuthNSvc)
	{
	case RPC_C_AUTHN_NONE:
		printf("Bez autentizace\n");
		break;
	case RPC_C_AUTHN_DCE_PRIVATE:
		printf("Autentizace soukromm klem DCE\n");
		break;
	case RPC_C_AUTHN_DCE_PUBLIC:
		printf("Autentizace veejnm klem DCE\n");
		break;
	case RPC_C_AUTHN_DEC_PUBLIC:
		printf("Autentizace veejnm klem DEC\n");
		break;
	case RPC_C_AUTHN_GSS_NEGOTIATE:
		printf("Poskytovatel podpory zabezpeen Snego\n");
		break;
	case RPC_C_AUTHN_WINNT:
		printf("NTLMSSP (Windows NT LAN Manager Security Support Provider)\n");
		break;
	case RPC_C_AUTHN_GSS_SCHANNEL:
		printf("Poskytovatel podpory zabezpeen SChannel\n");
		break;
	case RPC_C_AUTHN_DPA:
		printf("DPA  Momentln nen podporovno\n");
		break;
	case RPC_C_AUTHN_MSN:
		printf("MSN - Momentln nen podporovno\n");
		break;
	case RPC_C_AUTHN_GSS_KERBEROS:
		printf("Poskytovatel podpory zabezpeen Kerberos\n");
		break;
	case RPC_C_AUTHN_MQ:
		printf("MQ - Momentln nen podporovno\n");
		break;
	case RPC_C_AUTHN_DEFAULT:
		printf("Vchoz autentizace\n"); // sem bychom se nemli dostat 
		break;
	default:
		printf("Nov poskytovatel autentizace %d\n", AuthNSvc);
		break;
	}

	// nyn vpis autorizan sluby
	switch(AuthZSvc)
	{
	case RPC_C_AUTHZ_NONE:
		printf("Bez autorizace\n");
		break;
	case RPC_C_AUTHZ_NAME:
		printf("Autorizace podle hlavnho nzvu klienta\n");
		break;
	case RPC_C_AUTHZ_DCE:
		printf("Kontrola autorizace podle certifiktu s atributem oprvnn DCE\n");
		break;
	default:
		printf("Neznm poskytovatel autorizace %d\n", AuthZSvc);
		break;
	}

	// nzev serveru
	if(PriName != NULL)
		wprintf(L"Hlavn nzev: %s\n", PriName);

	// nyn rove autentizace 
	switch(AuthLevel)
	{
	case RPC_C_AUTHN_LEVEL_NONE:
		printf("rove autentizace = dn\n");
		break;
	case RPC_C_AUTHN_LEVEL_CONNECT:
		printf("rove autentizace = pipojen\n");
		break;
	case RPC_C_AUTHN_LEVEL_CALL:
		printf("rove autentizace = voln\n");
		break;
	case RPC_C_AUTHN_LEVEL_PKT:
		printf("rove autentizace = paketov\n");
		break;
	case RPC_C_AUTHN_LEVEL_PKT_INTEGRITY:
		printf("rove autentizace = integrita paket\n");
		break;
	case RPC_C_AUTHN_LEVEL_PKT_PRIVACY:
		printf("rove autentizace = soukrom paket\n");
		break;
	default:
		printf("Neznm rove autentizace %d\n", AuthLevel);
		break;
	}

	// rove zosobnn
	switch(ImpLevel)
	{
	case RPC_C_IMP_LEVEL_ANONYMOUS:
		printf("rove zosobnn = Anonymous\n");
		break;
	case RPC_C_IMP_LEVEL_IDENTIFY:
		printf("rove zosobnn = Identify\n");
		break;
	case RPC_C_IMP_LEVEL_IMPERSONATE:
		printf("rove zosobnn = Impersonate\n");
		break;
	case RPC_C_IMP_LEVEL_DELEGATE:
		printf("rove zosobnn = Delegate\n");
		break;
	default:
		printf("Neznm rove zosobnn %d\n", ImpLevel);
		break;
	}
}

void DumpServerSecuritySettings(ISecurityExample* pSecExample)
{
	DWORD AuthNSvc;
	DWORD AuthZSvc;
	DWORD AuthLevel;
	DWORD ImpLevel;
	HRESULT hr;

	hr = pSecExample->GetServerBlanket(&AuthNSvc, &AuthZSvc, &AuthLevel, &ImpLevel);

	if(hr == S_OK)
	{
		printf("\nInformace o zabezpeen serveru:\n");

		DumpSecuritySettings(AuthNSvc,
							 AuthZSvc, 
							 NULL, 
							 AuthLevel, 
							 ImpLevel);
	}
	else
	{
		printf("Nemohu zjistit nastaven bezpenosti serveru\n");
	}

}

void DumpClientSecuritySettings(IUnknown* pUnk)
{
	IClientSecurity* pClientSec;
	DWORD AuthNSvc;
	DWORD AuthZSvc;
	OLECHAR* PriName;
	DWORD AuthLevel;
	DWORD ImpLevel;

	HRESULT hr = pUnk->QueryInterface(IID_IClientSecurity, (void**)&pClientSec);

	printf("\nInformace o zabezpeen klienta:\n");
	if(hr == S_OK)
	{
		hr = pClientSec->QueryBlanket(pUnk,         //ukazatel rozhran
									  &AuthNSvc,    //autentizan sluba
									  &AuthZSvc,    //autorizan sluba
									  &PriName,  //hlavn nzev
									  &AuthLevel,   //rove autentizace
									  &ImpLevel,    //rove zosobnn
									  NULL,      //poadavek na autentizan informace 
									  NULL); //monosti

		if(hr == S_OK)
		{
			DumpSecuritySettings(AuthNSvc, AuthZSvc, PriName, AuthLevel, ImpLevel);
			CoTaskMemFree(PriName);
		}
		else
		{
			printf("Nemohu se dotzat na bezpenostn blanket klienta\n");
		}

		pClientSec->Release();
	}
	else
	{
		printf("Rozhran nepodporuje IClientSecurity\n");
	}


}

bool SetClientAuthLevel(IUnknown* pUnk, DWORD Level)
{
	IClientSecurity* pClientSec;
	DWORD AuthNSvc;
	DWORD AuthZSvc;
	DWORD AuthLevel;
	DWORD ImpLevel;

	HRESULT hr = pUnk->QueryInterface(IID_IClientSecurity, (void**)&pClientSec);

	// nejprve zjistit aktuln nastaven 
	if(hr == S_OK)
	{
		hr = pClientSec->QueryBlanket(pUnk,         //ukazatel rozhran
									  &AuthNSvc,    //autentizan sluba
									  &AuthZSvc,    //autorizan sluba
									  NULL,         //hlavn nzev
									  &AuthLevel,   //rove autentizace
									  &ImpLevel,    //rove zosobnn
									  NULL,      //poadavek na autentizan informace 
									  NULL); //monosti

		if(hr == S_OK)
		{
			// te si nastavme co chceme 
			hr = pClientSec->SetBlanket(pUnk, 
										AuthNSvc, 
										AuthZSvc, 
										NULL, 
										Level, 
										ImpLevel, 
										NULL, 
										EOAC_NONE);
			if(hr == S_OK)
				return true;
			else
				return false;
		}
		else
		{
			printf("Nemohu se dotzat na bezpenostn blanket klienta\n");
		}

		pClientSec->Release();
	}
	else
	{
		printf("Rozhran nepodporuje IClientSecurity\n");
	}

	return false;
}


//DCOM poaduje nzvy server v UNICODE  s funkc wmain si to usnadnme
int wmain(int argc, WCHAR* argv[])
{
	HRESULT hr;

	if(argc == 1)
	{
		wprintf(L"Voln: %s [nzev serveru]\n", argv[0]);
		return -1;
	}

	hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);

	if(hr != ERROR_SUCCESS)
	{
		printf("Nelze inicializovat?\n");
		return -1;
	}

	COSERVERINFO ServerInfo;

	//set up the COSERVERINFO struct
	ServerInfo.dwReserved1 = 0;  //vyhrazen pole mus bt rovna 0
	ServerInfo.dwReserved2 = 0;
	ServerInfo.pAuthInfo = NULL; //toto zmnme pozdji
	ServerInfo.pwszName = argv[1]; //nzev serveru

	//musme tak nastavit strukturu MULTI_QI - 
	MULTI_QI qi;

	qi.pIID = &IID_IUnknown; //ID rozhran, kter chceme otevt
	qi.pItf = NULL; //tm se vrt ukazatel na rozhran  inicializujeme na NULL
	qi.hr = 0;      //vrtme pro toto rozhran

	hr = CoCreateInstanceEx(CLSID_SecurityExample, //vytven CLSID 
							NULL,                 //nen to agregace
							CLSCTX_REMOTE_SERVER, //typ objektu
							&ServerInfo,          //informace o serveru
							1,                    //chceme jen 1 rozhran
							&qi);

	switch(hr)
	{
	case E_INVALIDARG:
		printf("Neplatn argument\n");
		break;

	case S_OK:
		break;

	case CO_S_NOTALLINTERFACES:
		printf("Ne vechna rozhran jsou k dispozici\n");
		break;

	case E_NOINTERFACE:
		printf("dn z poadovanch rozhran nen k dispozici\n");
		break;

	case REGDB_E_CLASSNOTREG:
		printf("Tda nen registrovna\n");
		break;
	default:
		printf("Neznm chyba %d (%x)\n", GetLastError(), GetLastError());
		break;
	}

	if(hr != S_OK || qi.hr != S_OK)
	{
		CoUninitialize();
		return -1;
	}

	IUnknown* pUnk = qi.pItf;
	ISecurityExample* pSecExample;

	//nyn mme rozhran na IUnknown  nateme smyslupln rozhran
	hr = pUnk->QueryInterface(IID_ISecurityExample, (void**)&pSecExample);

	if(hr == S_OK)
	{
		//nyn prozkoumme klientsk zabezpeen
		printf("Prvotn nastaven klientskho zabezpeen:\n");
		DumpClientSecuritySettings(pSecExample);

		//a jak to vypad na serveru?
		if(!SetClientAuthLevel(pSecExample, RPC_C_AUTHN_LEVEL_PKT_PRIVACY))
		{
			printf("Nemohu nastavit rove autentizace klienta\n");
		}
		else
		{
			printf("\nrove autentizace nastavena na Soukrom paket\n");
		}

		DumpServerSecuritySettings(pSecExample);

		pSecExample->Release();
	}
	else
	{
		printf("Nemohu nast ukazatel na ISecurityExample\n");
	}

	pUnk->Release();
	CoUninitialize();
	return 0;
}
