/*
  Phones.cpp
*/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <windows.h>
#include "phone.h"    // Hlavikov soubor vygeneroval kompiltor MIDL 

BOOL IsLRPC(void *ctx) {
    BOOL fAllowProtocol = FALSE;
    LPBYTE pBinding = NULL;

    if (RpcBindingToStringBinding(ctx,&pBinding) == RPC_S_OK) {

        LPBYTE  pProtSeq = NULL;
        // Zajmme se jen o sekvenci protokolu, take 
        // do ostatnch parametr meme zapsat NULL.

        if (RpcStringBindingParse(pBinding,
                                    NULL,
                                    &pProtSeq,
                                    NULL,
                                    NULL,
                                    NULL) == RPC_S_OK) {
            printf("Voln: %s\n",pProtSeq);
            
            // Kontrola, jestli byl klientsk poadavek 
            // podn pes LRPC.
            if (lstrcmpi((LPCTSTR)pProtSeq,"ncalrpc") == 0)
                fAllowProtocol = TRUE;

            if (pProtSeq)    
                RpcStringFree(&pProtSeq); 
        }

        if (pBinding)    
            RpcStringFree(&pBinding);
    }

    return fAllowProtocol;
}

RPC_STATUS RPC_ENTRY SecurityCallBack(RPC_IF_HANDLE idIF, void *ctx) {
    
    // Povolit jen provoz LRPC (je v komenti!)
    //if (!IsLRPC(ctx)) 
    //    RpcRaiseException(ERROR_PROTOCOL_UNREACHABLE);

    RPC_AUTHZ_HANDLE hPrivs;
    DWORD dwAuthn;

    RPC_STATUS status = RpcBindingInqAuthClient(
            ctx,
            &hPrivs,
            NULL,
            &dwAuthn,
            NULL,
            NULL);
                                        
    if (status != RPC_S_OK) {
        printf("RpcBindingInqAuthClient vrtilo: 0x%x\n", status);
        RpcRaiseException(ERROR_ACCESS_DENIED);
    }

    // Nyn zkontrolujeme rove autentizace.
    // Poadujeme nejmn autentizaci na rovni paket.
    if (dwAuthn < RPC_C_AUTHN_LEVEL_PKT) {
        printf("Klient se pokusil o slabou autentizaci.\n");
        RpcRaiseException(ERROR_ACCESS_DENIED);
    }

    return RPC_S_OK;
}

void Usage() {
    fprintf(stderr, "Voln:  Phones\n");
    fprintf(stderr, " -p sekvence_protokolu\n");
    fprintf(stderr, " -e koncov_bod\n");
    fprintf(stderr, " -m max_voln\n");
    fprintf(stderr, " -n min_voln\n");
    fprintf(stderr, " -f flag_wait_op\n");
    exit(1);
}

void __cdecl main(int argc, char *argv[]) {
    LPBYTE pszProtocolSequence = (LPBYTE)"ncacn_np";
    LPBYTE pszSecurity         = (LPBYTE)NULL;
    LPBYTE pszEndpoint         = (LPBYTE)"\\pipe\\phone";
    unsigned int cMinCalls     = 1;
    unsigned int cMaxCalls     = 20;
    unsigned int fDontWait     = FALSE;

    // Uivatel me potlait nastaven v pepnach pkazovho dku. 
    for (int i = 1; i < argc; i++) {
        if ((*argv[i] == '-') || (*argv[i] == '/')) {
            switch (tolower(*(argv[i]+1))) {
            case 'p':  
                pszProtocolSequence = (LPBYTE)argv[++i];
                break;
            case 'e':
                pszEndpoint = (LPBYTE)argv[++i];
                break;
            case 'm':
                cMaxCalls = (unsigned int) atoi(argv[++i]);
                break;
            case 'n':
                cMinCalls = (unsigned int) atoi(argv[++i]);
                break;
            case 'f':
                fDontWait = (unsigned int) atoi(argv[++i]);
                break;
            case 'h':
            case '?':
            default:
                Usage();
            }
        }
        else
            Usage();
    }

    RPC_STATUS status = RpcServerUseProtseqEp(pszProtocolSequence,
                                   cMaxCalls,
                                   pszEndpoint,
                                   pszSecurity);  
    printf("RpcServerUseProtseqEp vrtilo 0x%x\n", status);
    if (status)
        exit(status);

    status = RpcServerRegisterIfEx(phone_v1_0_s_ifspec,   
                                 NULL,
                                 NULL,
                                 0,
                                 RPC_C_LISTEN_MAX_CALLS_DEFAULT,
                                 SecurityCallBack);  
    printf("RpcServerRegisterIfEx vrtilo 0x%x\n", status);
    if (status)
        exit(status);

    LPBYTE szSPN = NULL;
    status = RpcServerInqDefaultPrincName(RPC_C_AUTHN_GSS_NEGOTIATE,
                                        &szSPN);
    printf("RpcServerInqDefaultPrincName vrtilo 0x%x (%s)\n", status,szSPN);
    if (status)
        exit(status);

    status = RpcServerRegisterAuthInfo(szSPN,
                                       RPC_C_AUTHN_GSS_NEGOTIATE,
                                       0,
                                       0);
    printf("RpcServerRegisterAuthInfo vrtilo 0x%x\n", status);
    if (status)
      exit(status);

    RPC_BINDING_VECTOR *pBindings = NULL;
    status = RpcServerInqBindings(&pBindings);
    printf("RpcServerInqBindings vrtilo 0x%x\n", status);
    if (status)
      exit(status);

    status = RpcEpRegister(phone_v1_0_s_ifspec,
                            pBindings,
                           NULL,
                           (LPBYTE)"Phone Application");
    printf("RpcEpRegister vrtilo 0x%x\n", status);
    if (status)
      exit(status);

    printf("Voln RpcServerListen\n");
    status = RpcServerListen(cMinCalls,
                             cMaxCalls,
                             fDontWait);
    printf("RpcServerListen vrtilo: 0x%x\n", status);
    if (status) 
        exit(status);

    if (fDontWait) {
        printf("Voln RpcMgmtWaitServerListen\n");
        status = RpcMgmtWaitServerListen();  
        printf("RpcMgmtWaitServerListen vrtilo: 0x%x\n", status);
       
        if (status) 
            exit(status);
    }

    if (szSPN)
        RpcStringFree(&szSPN);
}  

// Vzdlen funkce, Message
void Message(handle_t hPhone, unsigned char *szMsg) {

    if (RpcImpersonateClient(hPhone) != RPC_S_OK) {
        printf("Pevzet identity selhalo.\n");
        RpcRaiseException(ERROR_ACCESS_DENIED);
    }

    char szName[128+1];
    DWORD dwNameLen = 128;
    if (!GetUserName(szName,&dwNameLen))
        lstrcpy(szName,"Neznm uivatel");

    printf("Zprva od %s je: %s\n", 
           szName,szMsg);

    RpcRevertToSelf();
}

void __RPC_FAR * __RPC_USER midl_user_allocate(size_t len) {
    return(malloc(len));
}

void __RPC_USER midl_user_free(void __RPC_FAR * ptr) {
    free(ptr);
}

